==> demoniac-0.3.tar.gz: demoniac-0.3/libfx/dlgopt.cpp <==
/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * DlgOpt.cpp: options dialog box for GLDemoNiac.
 * Leto/calodox, 12.2000
 */

#include "opengl.h"
#include <gtk/gtk.h>

#include <assert.h>
#include "dlgopt.h"
#include "glmain.h"
#include "fxinstance.h"

DlgOpt::~DlgOpt() {

}

gint cbDlgReloadButton(GtkWidget *widget, gpointer *data) {
	DlgOpt *dlg = (DlgOpt *) data;

	for (unsigned int i=0; i<dlg->fxi->opts.size(); i++) {
		dlg->fxi->opts[i]->update();
	}
	
	dlg->fxi->finish();
	dlg->fxi->init();

	glMainptr->reDraw();
	return TRUE;
}

gint cbRenameInstance(GtkWidget *widget, gpointer *data) {
	DlgOpt *obj = (DlgOpt *) data;
	
	strncpy(obj->fxi->name, gtk_entry_get_text(GTK_ENTRY(widget)),
		MAX_FXINSTANCE_NAME_LENGTH);
	obj->fxi->name[MAX_FXINSTANCE_NAME_LENGTH-1]=0;
	glMainptr->renameInstance(obj->fxi);

	gtk_window_set_title(GTK_WINDOW(obj->window), obj->fxi->name);
	return TRUE;
}

void cbNowButton(GtkWidget *button, GtkWidget *spin) {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), glMainptr->getTimeSec());
}

void cbInitTime(GtkWidget *adj, DlgOpt *data) {
	int newVal = (int)(GTK_ADJUSTMENT(adj)->value * 1000.0);
	if (newVal != data->fxi->getInitTime()) 
		data->fxi->setInitTime((int)(GTK_ADJUSTMENT(adj)->value * 1000.0));
}
void cbStartTime(GtkWidget *adj, DlgOpt *data) {
	int newVal = (int)(GTK_ADJUSTMENT(adj)->value * 1000.0);
	if (newVal != data->fxi->getStartTime()) 
		data->fxi->setStartTime((int)(GTK_ADJUSTMENT(adj)->value * 1000.0));
}
void cbStopTime(GtkWidget *adj, DlgOpt *data) {
	int newVal = (int)(GTK_ADJUSTMENT(adj)->value * 1000.0);
	if (newVal != data->fxi->getStopTime()) 
		data->fxi->setStopTime((int)(GTK_ADJUSTMENT(adj)->value * 1000.0));
}

DlgOpt::DlgOpt(FXInstance *fxi) : 
	fxi(fxi), 
	ignore(false) 
{
}

void DlgOpt::build() {

	assert(fxi != 0);

	visible = false;

	// create the Gtk window
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), fxi->name);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", 
		GTK_SIGNAL_FUNC(Dlg::cbDeleteWindow), (Dlg *) this);

	// a scrolled area
	GtkWidget *scrWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrWin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	gtk_container_add(GTK_CONTAINER(window), scrWin);

	// and a vbox to pack things in
	vbox = gtk_vbox_new(FALSE, 3);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrWin), vbox);
	gtk_container_set_border_width( GTK_CONTAINER(vbox), 3 );

	// create a frame for generic instance options
	GtkWidget *instFrame = gtk_frame_new("Instance options");
	gtk_box_pack_start(GTK_BOX(vbox), instFrame, FALSE, FALSE, 3);
	GtkWidget *vboxInst = gtk_vbox_new(FALSE, 3);
	gtk_container_add(GTK_CONTAINER(instFrame), vboxInst);
	gtk_widget_show(instFrame);

	// create the "instance name" control
	GtkWidget *hboxName = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(vboxInst), hboxName, FALSE, FALSE, 0);
	gtk_widget_show(hboxName);
	GtkWidget *nameLabel = gtk_label_new("Instance name");
	gtk_box_pack_start(GTK_BOX(hboxName), nameLabel, FALSE, FALSE, 0);
	gtk_widget_show(nameLabel);
	GtkWidget *nameEntry = gtk_entry_new_with_max_length(MAX_FXINSTANCE_NAME_LENGTH);
	gtk_entry_set_text(GTK_ENTRY(nameEntry), fxi->name);
	gtk_box_pack_start(GTK_BOX(hboxName), nameEntry, TRUE, TRUE, 0);
	gtk_signal_connect(GTK_OBJECT(nameEntry), "changed",
			GTK_SIGNAL_FUNC(cbRenameInstance),
			this);
	gtk_widget_show(nameEntry);

	// create init/start/stop time spinners

	initSpin = addTimeSpinner(GTK_BOX(vboxInst), "Initialisation time", GTK_SIGNAL_FUNC(cbInitTime));
	startSpin = addTimeSpinner(GTK_BOX(vboxInst), "Start time", GTK_SIGNAL_FUNC(cbStartTime));
	stopSpin = addTimeSpinner(GTK_BOX(vboxInst), "Stop time", GTK_SIGNAL_FUNC(cbStopTime));
	updateTimeSpinners();

	// create the frame for initialization options
	GtkWidget *initFrame = gtk_frame_new("Initialisation options");
	gtk_box_pack_start(GTK_BOX(vbox), initFrame, FALSE, FALSE, 5);
	gtk_widget_show(initFrame);
	// create the VBox for initialization options
	initVBox = gtk_vbox_new(FALSE, 3);
	gtk_container_set_border_width( GTK_CONTAINER(initVBox), 5 );
	gtk_container_add(GTK_CONTAINER(initFrame), initVBox);
	gtk_widget_show(initVBox);

	// create the frame for run time options
	GtkWidget *rtFrame = gtk_frame_new("Run time options");
	gtk_box_pack_start(GTK_BOX(vbox), rtFrame, FALSE, FALSE, 5);
	gtk_widget_show(rtFrame);
	// create the VBox for run time options
	rtVBox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(rtFrame), rtVBox);
	gtk_container_set_border_width( GTK_CONTAINER(rtVBox), 3 );

	// add the fx options in the window
	for (unsigned int i=0; i<fxi->opts.size(); i++) {
		if (fxi->opts[i]->flags & OPTION_REINIT)
			fxi->opts[i]->draw(GTK_BOX(initVBox));
		else
			fxi->opts[i]->draw(GTK_BOX(rtVBox));
	}

	GtkWidget *reload = gtk_button_new_with_label("Reload");
	gtk_box_pack_start(GTK_BOX(vbox), reload, FALSE, FALSE, 5);
	gtk_signal_connect(GTK_OBJECT(reload), "clicked",
		GTK_SIGNAL_FUNC(cbDlgReloadButton), this);

	// we're done. Show the result. 
	gtk_widget_show_all(scrWin);
}

const char *DlgOpt::getName() {
	return (const char *) fxi->name;
}

void DlgOpt::show() {

	if (window) {
		if (!visible) {
			gtk_widget_show(window);
			visible = true;

			int x = vbox->allocation.width + 22;
			int y = vbox->allocation.height + 22;

			x = x > 640 ? 640 : x;
			y = y > 600 ? 600 : y;

			gtk_window_set_default_size(GTK_WINDOW(window), x, y);
		} else {
			gdk_window_raise(GTK_WIDGET(window)->window);
		}
	}
}

void DlgOpt::updateGui() {
	for (unsigned int i=0; i<fxi->opts.size(); i++) {
		fxi->opts[i]->updateGui();
	}
}

void DlgOpt::updateTimeSpinners() {
	if (this && !this->ignore) {
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(initSpin), ((gfloat)fxi->getInitTime())/ 1000.0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(startSpin), ((gfloat)fxi->getStartTime())/1000.0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(stopSpin), ((gfloat)fxi->getStopTime())/1000.0);
	}
}
	
GtkSpinButton *DlgOpt::addTimeSpinner(GtkBox *box, char *label, GtkSignalFunc callback) {

	// hbox
	GtkWidget *hbox= gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

	//label
	GtkWidget *lab= gtk_label_new(label); 
	gtk_box_pack_start(GTK_BOX(hbox), lab, FALSE, FALSE, 0);

	// adjustment & spin_button
	GtkAdjustment *adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, FLT_MAX, 1, 10, 0));
	GtkWidget *spin = gtk_spin_button_new(adj, 1,  3);
	gtk_signal_connect(GTK_OBJECT(adj), "value_changed", 
		GTK_SIGNAL_FUNC(callback), this);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);

	// 'now' button
	GtkWidget *button = gtk_button_new_with_label("now"); 
	gtk_signal_connect(GTK_OBJECT(button), "clicked", 
		GTK_SIGNAL_FUNC(cbNowButton), spin);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);

	gtk_widget_show_all(hbox);

	return GTK_SPIN_BUTTON(spin);
}

==> demoniac-0.4.tar.gz: demoniac-0.4/libfx/dlgopt.cpp <==
/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * DlgOpt.cpp: options dialog box for GLDemoNiac.
 * Leto/calodox, 12.2000
 */

#include "opengl.h"
#include <gtk/gtk.h>

#include <assert.h>
#include "dlgopt.h"
#include "glmain.h"
#include "fxinstance.h"

DlgOpt::~DlgOpt() {

}

gint cbDlgReloadButton(GtkWidget *widget, gpointer *data) {
	DlgOpt *dlg = (DlgOpt *) data;

	for (unsigned int i=0; i<dlg->fxi->opts.size(); i++) {
		dlg->fxi->opts[i]->update();
	}
	
	dlg->fxi->finish();
	dlg->fxi->init();

	glMainptr->reDraw();
	return TRUE;
}

/*
gint cbRenameInstance(GtkWidget *widget, gpointer *data) {
	DlgOpt *obj = (DlgOpt *) data;
	
	obj->fxi->setName(gtk_entry_get_text(GTK_ENTRY(widget)));
	glMainptr->renameInstance(obj->fxi);

	gtk_window_set_title(GTK_WINDOW(obj->window), obj->fxi->name);
	return TRUE;
}
*/

void cbNowButton(GtkWidget *button, GtkWidget *spin) {
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), glMainptr->getTimeSec());
}

void cbInitTime(GtkWidget *adj, DlgOpt *data) {
	int newVal = (int)(GTK_ADJUSTMENT(adj)->value * 1000.0);
	if (newVal != data->fxi->initTime)  {
		data->fxi->setInitTime((int)(GTK_ADJUSTMENT(adj)->value * 1000.0));
		data->updateGui();
	}
}
void cbStartTime(GtkWidget *adj, DlgOpt *data) {
	int newVal = (int)(GTK_ADJUSTMENT(adj)->value * 1000.0);
	if (newVal != data->fxi->startTime) {
		data->fxi->setStartTime((int)(GTK_ADJUSTMENT(adj)->value * 1000.0));
		data->updateGui();
	}
}
void cbStopTime(GtkWidget *adj, DlgOpt *data) {
	int newVal = (int)(GTK_ADJUSTMENT(adj)->value * 1000.0);
	if (newVal != data->fxi->duration +  data->fxi->startTime) {
		data->fxi->setDuration(newVal - data->fxi->startTime);
		data->updateGui();
	}
}

DlgOpt::DlgOpt(FXInstance *fxi) : 
	fxi(fxi), 
	ignore(false) 
{
}

void DlgOpt::build() {

	assert(fxi != 0);

	visible = false;

	// create the Gtk window
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), fxi->name);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", 
		GTK_SIGNAL_FUNC(Dlg::cbDeleteWindow), (Dlg *) this);

	// a scrolled area
	GtkWidget *scrWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrWin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	gtk_container_add(GTK_CONTAINER(window), scrWin);

	// and a vbox to pack things in
	vbox = gtk_vbox_new(FALSE, 3);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrWin), vbox);
	gtk_container_set_border_width( GTK_CONTAINER(vbox), 3 );

	// create the frame for initialization options
	GtkWidget *initFrame = gtk_frame_new("Initialisation options");
	gtk_box_pack_start(GTK_BOX(vbox), initFrame, FALSE, FALSE, 5);
	gtk_widget_show(initFrame);
	// create the VBox for initialization options
	initVBox = gtk_vbox_new(FALSE, 3);
	gtk_container_set_border_width( GTK_CONTAINER(initVBox), 5 );
	gtk_container_add(GTK_CONTAINER(initFrame), initVBox);
	gtk_widget_show(initVBox);

	// create the frame for run time options
	GtkWidget *rtFrame = gtk_frame_new("Run time options");
	gtk_box_pack_start(GTK_BOX(vbox), rtFrame, FALSE, FALSE, 5);
	gtk_widget_show(rtFrame);
	// create the VBox for run time options
	rtVBox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(rtFrame), rtVBox);
	gtk_container_set_border_width( GTK_CONTAINER(rtVBox), 3 );

	// add the fx options in the window
	for (unsigned int i=0; i<fxi->opts.size(); i++) {
		if (fxi->opts[i]->flags & OPTION_REINIT)
			fxi->opts[i]->draw(GTK_BOX(initVBox));
		else
			fxi->opts[i]->draw(GTK_BOX(rtVBox));
	}

	GtkWidget *reload = gtk_button_new_with_label("Reload");
	gtk_box_pack_start(GTK_BOX(vbox), reload, FALSE, FALSE, 5);
	gtk_signal_connect(GTK_OBJECT(reload), "clicked",
		GTK_SIGNAL_FUNC(cbDlgReloadButton), this);

	// we're done. Show the result. 
	gtk_widget_show_all(scrWin);
}

const char *DlgOpt::getName() {
	return (const char *) fxi->name;
}

void DlgOpt::show() {

	if (window) {
		if (!visible) {
			gtk_widget_show(window);
			visible = true;

			int x = vbox->allocation.width + 22;
			int y = vbox->allocation.height + 22;

			x = x > 640 ? 640 : x;
			y = y > 600 ? 600 : y;

			gtk_window_set_default_size(GTK_WINDOW(window), x, y);
		} else {
			gdk_window_raise(GTK_WIDGET(window)->window);
		}
	}
}

void DlgOpt::updateGui() {
	for (unsigned int i=0; i<fxi->opts.size(); i++) {
		fxi->opts[i]->updateGui();
	}
}

void DlgOpt::updateTimeSpinners() {
	if (this && !this->ignore) {
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(initSpin), ((gfloat)fxi->initTime)/ 1000.0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(startSpin), ((gfloat)fxi->startTime)/1000.0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(stopSpin), ((gfloat)fxi->duration + fxi->startTime)/1000.0);
	}
}
	
GtkSpinButton *DlgOpt::addTimeSpinner(GtkBox *box, char *label, GtkSignalFunc callback) {

	// hbox
	GtkWidget *hbox= gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(box, hbox, FALSE, FALSE, 0);

	//label
	GtkWidget *lab= gtk_label_new(label); 
	gtk_box_pack_start(GTK_BOX(hbox), lab, FALSE, FALSE, 0);

	// adjustment & spin_button
	GtkAdjustment *adj = GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, FLT_MAX, 1, 10, 0));
	GtkWidget *spin = gtk_spin_button_new(adj, 1,  3);
	gtk_signal_connect(GTK_OBJECT(adj), "value_changed", 
		GTK_SIGNAL_FUNC(callback), this);
	gtk_box_pack_start(GTK_BOX(hbox), spin, TRUE, TRUE, 0);

	// 'now' button
	GtkWidget *button = gtk_button_new_with_label("now"); 
	gtk_signal_connect(GTK_OBJECT(button), "clicked", 
		GTK_SIGNAL_FUNC(cbNowButton), spin);
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);

	gtk_widget_show_all(hbox);

	return GTK_SPIN_BUTTON(spin);
}
